package Chooser::Checks::cidr;

use warnings;
use strict;
use base 'Error::Helper';

=head1 NAME

Chooser::Check::cidr - This provides the cidr check for Chooser.

=head1 VERSION

Version 0.0.0

=cut

our $VERSION = '0.0.0';

=head1 METHODS

=head2 new

This initiates the object.

No arguments are taken and this will
never error.

    my $check=Chooser::Check::cidr->new;

=cut

sub new{

	my $self={
		error=>undef,
		perror=>undef,
		errorString=>undef,
		errorExtra=>{0
			flags=>{
				'1'=>'noIF',
			},
		},
	};
	bless $self;

	return $self;
}

=head2 test

This runs the test.

=cut

sub test{
	my $self=$_[0];
	my %args= %{$_[1]};

	if ( ! defined( $args{'if'} ) ){
		$self->{error}=1;
		$self->{errorString}='No interface specified';
		$self->warn;
		return undef;
	}

	if ( ! defined( $args{'cidr'} ) ){
		$self->{error}=2;
		$self->{errorString}='No subnet specified';
		$self->warn;
		return undef;
	}

	my $cidr = Net::CIDR::Lite->new;
	
	$cidr->add($args{cidr});
	
	my $socket = IO::Socket::INET->new(Proto=>'udp');
	
	my @iflist=$socket->if_list;
	
	#if a interface is not specified, make sure it exists
	if(defined($args{if})){
		my $iflistInt=0;#used for intering through @iflist
		while(defined($iflist[$iflistInt])){
			#checks if this is the interface in question
			if($iflist[$iflistInt] eq $args{if}){
				#gets the address
				my $address=$socket->if_addr($args{if});
				#if the interface does not have a address, don't check it
				if(defined($address)){
					#checks this address is with in this cidr
					if ($cidr->find($address)){
						return 1;
					}
				}
			}
			
			$iflistInt++;
		}
		
		#if a specific IP is defined and it reaches this point, it means it was now found
		return "0";
	}

	#if a interface is not specified, make sure it exists
	my $iflistInt=0;#used for intering through @iflist
	while(defined($iflist[$iflistInt])){
		#gets the address
		my $address=$socket->if_addr($iflist[$iflistInt]);
		#if the interface does not have a address, don't check it
		if(defined($address)){
			#checks this address is with in this cidr
			if ($cidr->find($address)){
				return 1;
			}
		}
		$iflistInt++;
	}
	
	return "0";

}

=head1 ERROR CODES/FLAGS

=head2 1, noIF

No interface specified. 'if' value not defined.

=head2 2, noIF

No subnet specified. 'cidr' value not defined.

=head1 BUGS

Please report any bugs or feature requests to C<bug-chooser at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Chooser>.  I will be notified, and then you'll
automatically be notified of progress on your bug as I make changes.

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc Chooser::Checks::cidr

You can also look for information at:

=over 4

=item * RT: CPAN's request tracker

L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Chooser>

=item * AnnoCPAN: Annotated CPAN documentation

L<http://annocpan.org/dist/Chooser>

=item * CPAN Ratings

L<http://cpanratings.perl.org/d/Chooser>

=item * Search CPAN

L<http://search.cpan.org/dist/Chooser>

=back

=head1 COPYRIGHT & LICENSE

Copyright 2012 Zane C. Bowers-Hadley, all rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.


=cut

1; # End of Chooser::Checks::cidr
