package Chooser::Checks::sslcert;

use warnings;
use strict;
use base 'Error::Helper';

=head1 NAME

Chooser::Check::sslcert - This provides the sslcert check for Chooser.

=head1 VERSION

Version 0.0.0

=cut

our $VERSION = '0.0.0';

=head1 METHODS

=head2 new

This initiates the object.

No arguments are taken and this will
never error.

    my $check=Chooser::Check::defgateway->new;

=cut

sub new{

	my $self={
		error=>undef,
		perror=>undef,
		errorString=>undef,
		errorExtra=>{
			flags=>{
				'1'=>'noHost',
				'2'=>'noPort',
			},
		},
	};
	bless $self;

	return $self;
}

=head2 test

This runs the test.

=cut

sub test{
	my $self=$_[0];
	my %args= %{$_[1]};

	if ( ! defined( $args{host} ) ){
		$self->{error}=1;
		$self->{errorString}='No host specified';
		$self->warn;
		return undef;
	}

	if ( ! defined( $args{port} ) ){
		$self->{error}=2;
		$self->{errorString}='No port specified';
		$self->warn;
		return undef;
	}
	
	my $client=IO::Socket::SSL->new( $args{host}.':'.$args{port},
									 SSL_version=>$args{version},
									 SSL_cipher_list=>$args{cipher_list},
									 SSL_ca_file=>$args{ca_file},
									 SSL_ca_path=>$args{ca_path},
									 SSL_crl_file=>$args{crl_file},
									 SSL_verify_mode=>$args{verify_mode},
									 SSL_verifycn_name=>$args{verifycn_name},
									 SSL_verifycn_scheme=>$args{verifycn_scheme},
		);
	
	if (!$client) {
		return 0;
	}
	
	my $certinfo=$client->dump_peer_certificate;
	
	# 0 is the subject
	# 1 is the issuer
	my @certinfoA=split(/\n/, $certinfo);
	
	#process the subject
	my $subject=$certinfoA[0];
	$subject=~s/^Subject\ Name\:\ //;
	
	if ($args{subject} ne $subject) {
		return 0
	}
	
	#process the issuer
	if (defined($args{issuer})) {
		my $issuer=$certinfoA[1];
		$issuer=s/^Issuer\ \ Name\:\ //g;
		
		if ($args{issuer} ne $issuer) {
			return 0
		}
	}
	
	return "0";

}

=head1 ERROR CODES/FLAGS

=head2 1, noHost

No host specified.

=head2 2, noPort

No port specified.

=head1 BUGS

Please report any bugs or feature requests to C<bug-chooser at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Chooser>.  I will be notified, and then you'll
automatically be notified of progress on your bug as I make changes.

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc Chooser::Checks::sslcert

You can also look for information at:

=over 4

=item * RT: CPAN's request tracker

L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Chooser>

=item * AnnoCPAN: Annotated CPAN documentation

L<http://annocpan.org/dist/Chooser>

=item * CPAN Ratings

L<http://cpanratings.perl.org/d/Chooser>

=item * Search CPAN

L<http://search.cpan.org/dist/Chooser>

=back

=head1 COPYRIGHT & LICENSE

Copyright 2012 Zane C. Bowers-Hadley, all rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.


=cut

1; # End of Chooser::Checks::sslcert
