package Chooser::Parse;

use warnings;
use strict;
use base 'Error::Helper';
use Text::NeatTemplate;
use Sys::Hostname;

=head1 NAME

Chooser::Parse - Parses a string for Chooser.

=head1 VERSION

Version 0.0.0

=cut

our $VERSION = '0.0.0';

=head1 SYNOPSIS

=head1 METHODS

=head2 new

This initiates the object.

No arguments are taken and this will
never error.

    my $cp=Chooser::Parser->new;

=cut

sub new{

	my $self={
		error=>undef,
		perror=>undef,
		errorString=>undef,
		errorExtra=>{
			flags=>{
				1=>'NoLine',
				2=>'NoCheck',
				3=>'NoExpect',
				4=>'NoChooser',
			},
		},
	};
	bless $self;

	return $self;
}

=head2 argstohash

This parses the args and returns a hash.

Two values are taken. The first is the
args string and it is required. The second
is a variable hash that will be passed to
L<Text::NeatTemplate>.

A blank string returns a empty hash.

    my %args=$cp->argstohash( $argsString );
    if ( $cp->error ){
        warn('error:'.$cp->error.': '.$cp->errorString);
    }

=cut

sub argstohash{
	my $self=$_[0];
	my $argsString=$_[1];
	my %vars;
	if ( ! defined( $_[2] ) ){
		%vars=%{$_[2]};
	}

	if ( ! $self->errorblank ){
		return undef;
	}

	my %args;

	if (
		(! defined( $argsString ) ) ||
		( $argsString eq '' ) ||
		( $argsString =~ /^[ \t]*$/ )
		){
		return %args;
	}

	my @argsStringSplit=split(/\|/, $argsString);

	#puts the hash together
	my %targs;
	$targs{hostname}=hostname;
	$targs{pipe}='|';
	$targs{newline}="\n";
	#adds %ENV stuff
	my @keys=keys(%ENV);
	my $keysInt=0;
	while (defined($keys[$keysInt])) {
		$targs{'ENV'.$keys[$keysInt]}=$ENV{$keys[$keysInt]};

		$keysInt++;
	}
	#add the var stuff
	@keys=keys(%vars);
	$keysInt=0;
	while (defined($keys[$keysInt])) {
		$targs{'VAR'.$keys[$keysInt]}=$vars{$keys[$keysInt]};

		$keysInt++;
	}	

	#puts a hash of arguements together
	my $argInt=0; #starting at 2 as it is the next in the line
	while(defined($argsStringSplit[$argInt])){
		my @argsplit=split(/=/, $argsStringSplit[$argInt], 2);
		
		#runs the template over it
		my $tobj = Text::NeatTemplate->new();
		$args{$argsplit[0]}=$tobj->fill_in(
										   data_hash=>\%targs,
										   template=>$argsplit[1],
										   );
		
		$argInt++;
	}
	
	return %args;
}

=head2 parseChooser

This parses a Chooser string.

One argument is taken and it is the Chooser
string to parse.

=head3 returned hash

=head4 vars

This is a hash of variables.

=head4 lines

This is a array of hashes as parsed by parseLine.

    my %parsed=$cp->parseChooser( $chooserString );
    if ( $cp->error ){
        warn('error:'.$cp->error.': '.$cp->errorString);
    }

=cut

sub parseChooser{
	my $self=$_[0];
	my $chooser=$_[1];

	if ( ! $self->errorblank ){
		return undef;
	}

	if ( ! defined( $chooser ) ){
		$self->{error}=4;
		$self->{errorString}='No chooser specified';
		$self->warn;
		return undef;
	}
	
	my @rawlines=split(/\n/, $chooser);

	my @lines;
	my %vars;

	my $int=0;
	while( defined( $rawlines[$int] ) ){
		my %line=$self->parseLine( $rawlines[$int], \%vars );
		if ( $self->error ){
			$self->warnString('parseLine failed for line "'.$int.'"');
			return undef;
		}

		if ( defined( $line{vars} ) ){
			%vars=%vars;
		}else{
			push( @lines, \%line );
		}

		$int++;
	}

	my %toreturn;
	$toreturn{lines}=\@lines;
	$toreturn{vars}=\%vars;

	return %toreturn;
}

=head2 parseLine

This parses the specified line.

Two arguments are taken. The first is
the line to parse and is required. The
second is the variables hash that will
be passed to L<Text::NeatTemplate>.

The returned value is and the keys are
as below.

=head3 return hash

=head4 check

This is the specified check.

=head4 expect

This is the expect value.

=head4 value

This is the value part of the line.

=head4 weight

This is the weight of the value.

=head4 args

This is a hash containing the args.

=head4 vars

If this is defined, then none of the
other keys will be present.

This will contain a updated vars hash.

=cut

sub parseLine{
	my $self=$_[0];
	my $line=$_[1];
	my %vars;
	if ( ! defined( $_[2] ) ){
		%vars=%{$_[2]};
	}

	if ( ! $self->errorblank ){
		return undef;
	}

	if ( ! defined( $line ) ){
		$self->{error}=1;
		$self->{errorString}='No line specified';
		$self->warn;
		return undef;
	}

	chomp( $line );
	
	#this is what will be returned
	my %toreturn;
	
	#parses it if it is a variable
	if ($line =~ /^\$/) {
		$line=~s/^\$//;
		my ($variable, $data)=split(/\=/, $line, 2);
		
		if (defined($data)) {
			#puts the hash together
			my %targs;
			$targs{hostname}=hostname;
			$targs{pipe}='|';
			$targs{newline}="\n";
			#adds %ENV stuff
			my @keys=keys(%ENV);
			my $keysInt=0;
			while (defined($keys[$keysInt])) {
				$targs{'ENV'.$keys[$keysInt]}=$ENV{$keys[$keysInt]};
				
				$keysInt++;
			}
			@keys=keys(%vars);
			$keysInt=0;
			while (defined($keys[$keysInt])) {
				$targs{'VAR'.$keys[$keysInt]}=$vars{$keys[$keysInt]};
				
				$keysInt++;
			}
			
			#runs the template over it
			my $tobj = Text::NeatTemplate->new;
			$vars{$variable}=$tobj->fill_in(
				data_hash=>\%targs,
				template=>$data,
				);
		}

		$toreturn{vars}=\%vars;
		return %toreturn;
	}

	my ($check, $restofline)=split(/\|/, $line, 2);
	(my $expect, $restofline)=split(/\|/, $restofline, 2);
	(my $value, $restofline)=split(/\|/, $restofline, 2);
	(my $weight, $restofline)=split(/\|/, $restofline, 2);
	(my $argsString, $restofline)=split(/\|/, $restofline, 2);
	my %args=argstohash($argsString,\%vars);
	
	if (!defined($weight)){
		$weight=0
	}
	
	#make sure we have a check
	if (
		( ! defined( $check ) ) ||
		( $check eq '' ) ||
		( $check =~ /^[ \t]$/ )
		){
		$self->{error}=2;
		$self->{errorString}='No check specified';
		$self->warn;
		return undef;
	}
	$check=~s/^[ \t]*//;
	$check=~s/[ \t]*$//;
	$toreturn{'check'}=$check;
	
	#make sure we have a expect
	if (
		( ! defined( $expect ) ) ||
		( $expect eq '' ) ||
		( $expect =~ /^[ \t]$/ )
		){
		$self->{error}=3;
		$self->{errorString}='No expect specified';
		$self->warn;
		return undef;
	}
	$expect=~s/^[ \t]*//;
	$expect=~s/[ \t]*$//;
	$toreturn{'expect'}=$expect;

	#make sure we have a numeric weight
	$weight=~s/^[ \t]*//;
	$weight=~s/[ \t]*$//;
	if (
		( ! defined( $expect ) ) ||
		( $expect eq '' ) ||
		( $weight !~ /^[0123456789]*$/ )
		){
		$self->{error}=4;
		$self->{errorString}='Improper weight specified';
		$self->warn;
		return undef;
	}
	$toreturn{'weight'}=$weight;

	$toreturn{'args'}=\%args;

	return %toreturn;
}

=head1 ERROR CODES/FLAGS

=head2 1, NoLine

No line specified.

=head2 2, NoCheck

No check specified.

=head2 3, NoExpect

No expect specified.

=head2 4, NoChooser

No chooser specified.

=head1 BUGS

Please report any bugs or feature requests to C<bug-chooser at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Chooser>.  I will be notified, and then you'll
automatically be notified of progress on your bug as I make changes.

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc Chooser


You can also look for information at:

=over 4

=item * RT: CPAN's request tracker

L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Chooser>

=item * AnnoCPAN: Annotated CPAN documentation

L<http://annocpan.org/dist/Chooser>

=item * CPAN Ratings

L<http://cpanratings.perl.org/d/Chooser>

=item * Search CPAN

L<http://search.cpan.org/dist/Chooser>

=back



=head1 COPYRIGHT & LICENSE

Copyright 2012 Zane C. Bowers, all rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.


=cut

1; # End of Chooser::Parse
